searchData={"items":[{"type":"module","title":"Faulty","doc":"En Elixir-based  error tracking solution.\n\nThe main objectives behind this project are:\n\n* Provide a basic free error tracking solution: because tracking errors in\nyour application should be a requirement for almost any project, and helps to\nprovide quality and maintenance to your project.","ref":"Faulty.html"},{"type":"module","title":"Requirements - Faulty","doc":"Faulty requires Elixir 1.15+","ref":"Faulty.html#module-requirements"},{"type":"module","title":"Integrations - Faulty","doc":"We currently include integrations for what we consider the basic stack of\nan application: Phoenix, Plug, Oban and Quantum.\n\nIf you want to manually report an error, you can use the `Faulty.message/2` function.","ref":"Faulty.html#module-integrations"},{"type":"module","title":"Context - Faulty","doc":"Aside from the information about each exception (kind, message, stack trace...)\nwe also store contexts.\n\nContexts are arbitrary maps that allow you to store extra information about an\nexception to be able to reproduce it later.\n\nEach integration includes a default context with useful information they\ncan gather, but aside from that, you can also add your own information. You can\ndo this in a per-process basis or in a per-call basis (or both).\n\nThere are some requirements on the type of data that can be included in the\ncontext, so we recommend taking a look at `set_context/1` documentation.\n\n**Per process**\n\nThis allows you to set a general context for the current process such as a Phoenix\nrequest or an Quantum or Oban job. For example, you could include the following code in your\nauthentication Plug to automatically include the user ID in any error that is\ntracked during the Phoenix request handling.\n\n```elixir\nFaulty.set_context(%{user_id: conn.assigns.current_user.id})\n```\n\n**Per call**\n\nAs we had seen before, you can use `Faulty.message/2` to manually report an\nerror. The second parameter of this function is optional and allows you to include\nextra context that will be tracked along with the error.","ref":"Faulty.html#module-context"},{"type":"type","title":"Faulty.context/0","doc":"A map containing the relevant context for a particular error.","ref":"Faulty.html#t:context/0"},{"type":"type","title":"Faulty.exception/0","doc":"An `Exception` or a `{kind, payload}` tuple compatible with `Exception.normalize/3`.","ref":"Faulty.html#t:exception/0"},{"type":"function","title":"Faulty.get_context/0","doc":"Obtain the context of the current process.","ref":"Faulty.html#get_context/0"},{"type":"function","title":"Faulty.message/2","doc":"Reports a message to be stored.\n\nReturns `:ok` stored or `:noop` if the Faulty is disabled by\nconfiguration the exception has not been stored.\n\nThis allows you to store a message or exception manually,\nthe stacktrace will be added automatically.\n\n```elixir\nFaulty.message(\"Invalid user or password\", %{login: login, password: password})\n\nFaulty.message({ArgumentError, \"Invalid user or password\"}, %{login: login, password: password})\n```","ref":"Faulty.html#message/2"},{"type":"function","title":"Exceptions - Faulty.message/2","doc":"Exceptions can be passed in three different forms:\n\n* A binary: This will be stored as an `ErlangError`.\n\n* An exception struct: the module of the exception is stored along with\nthe exception message.\n\n* A `{kind, exception}` tuple in which case the information is converted to\nan Elixir exception (if possible) and stored.\n\nThis function can also be used to test your setup.","ref":"Faulty.html#message/2-exceptions"},{"type":"function","title":"Faulty.report/3","doc":"Report an exception to be stored.\n\nReturns `:ok` stored or `:noop` if the Faulty is disabled by\nconfiguration the exception has not been stored.\n\nAside from the exception, it is expected to receive the stack trace and,\noptionally, a context map which will be merged with the current process\ncontext.\n\nKeep in mind that errors that occur in Phoenix controllers, Phoenix LiveViews\nand Quantum and Oban jobs are automatically reported. You will need this function only if you\nwant to report custom errors.\n\n```elixir\ntry do\n  # your code\ncatch\n  e ->\n    Faulty.report(e, __STACKTRACE__)\nend\n```","ref":"Faulty.html#report/3"},{"type":"function","title":"Exceptions - Faulty.report/3","doc":"Exceptions can be passed as:\n\n* An exception struct: the module of the exception is stored along with\nthe exception message.\n\n* A `{kind, exception}` tuple in which case the information is converted to\nan Elixir exception (if possible) and stored.","ref":"Faulty.html#report/3-exceptions"},{"type":"function","title":"Faulty.set_context/1","doc":"Sets the current process context.\n\nThe given context will be merged into the current process context. The given context\nmay override existing keys from the current process context.","ref":"Faulty.html#set_context/1"},{"type":"function","title":"Context depth - Faulty.set_context/1","doc":"You can store context on more than one level of depth, but take into account\nthat the merge operation is performed on the first level.\n\nThat means that any existing data on deep levels for he current context will\nbe replaced if the first level key is received on the new contents.","ref":"Faulty.html#set_context/1-context-depth"},{"type":"function","title":"Content serialization - Faulty.set_context/1","doc":"The content stored on the context should be serializable using the JSON library\nused by the application (usually `Jason`), so it is rather recommended to use\nprimitive types (strings, numbers, booleans...).\n\nIf you still need to pass more complex data types to your context, please test\nthat they can be encoded to JSON or storing the errors will fail. In the case\nof `Jason` that may require defining an Encoder for that data type if not\nincluded by default.","ref":"Faulty.html#set_context/1-content-serialization"},{"type":"module","title":"Faulty.Error","doc":"Schema to store an error or exception recorded by Faulty.\n\nIt stores a kind, reason and source code location to generate a unique\nfingerprint that can be used to avoid duplicates.\n\nThe fingerprint currently does not include the reason itself because it can\ncontain specific details that can change on the same error depending on\nruntime conditions.","ref":"Faulty.Error.html"},{"type":"function","title":"Faulty.Error.has_source_info?/1","doc":"Returns if the Error has information of the source or not.\n\nErrors usually have information about in which line and function occurred, but\nin some cases (like an Oban job ending with `{:error, any()}`) we cannot get\nthat information and no source is stored.","ref":"Faulty.Error.html#has_source_info?/1"},{"type":"type","title":"Faulty.Error.t/0","doc":"","ref":"Faulty.Error.html#t:t/0"},{"type":"behaviour","title":"Faulty.Filter","doc":"Behaviour for sanitizing & modifying the error context before it's saved.\n\n    defmodule MyApp.ErrorFilter do\n      @behaviour Faulty.Filter\n\n      @impl true\n      def sanitize(context) do\n        context # Modify the context object (add or remove fields as much as you need.)\n      end\n    end\n\nOnce implemented, include it in the Faulty configuration:\n\n  config :faulty, filter: MyApp.Filter\n\nWith this configuration in place, the Faulty will call `MyApp.Filter.sanitize/1` to get a context before\nsaving error occurrence.\n\n> #### A note on performance {: .warning}\n>\n> Keep in mind that the `sanitize/1` will be called in the context of the Faulty itself.\n> Slow code will have a significant impact in the Faulty performance. Buggy code can bring\n> the Faulty process down.","ref":"Faulty.Filter.html"},{"type":"callback","title":"Faulty.Filter.sanitize/1","doc":"This function will be given an error context to inspect/modify before it's saved.","ref":"Faulty.Filter.html#c:sanitize/1"},{"type":"behaviour","title":"Faulty.Ignorer","doc":"Behaviour for ignoring errors.\n\nThe Faulty tracks every error that happens in your application. In certain cases you may\nwant to ignore some errors and don't track them. To do so you can implement this behaviour.\n\n    defmodule MyApp.ErrorIgnorer do\n      @behaviour Faulty.Ignorer\n\n      @impl true\n      def ignore?(error = %Faulty.Error{}, context) do\n        # return true if the error should be ignored\n      end\n    end\n\nOnce implemented, include it in the Faulty configuration:\n\n    config :faulty, ignorer: MyApp.ErrorIgnorer\n\nWith this configuration in place, the Faulty will call `MyApp.ErrorIgnorer.ignore?/2` before\ntracking errors. If the function returns `true` the error will be ignored and won't be tracked.\n\n> #### A note on performance {: .warning}\n>\n> Keep in mind that the `ignore?/2` will be called in the context of the Faulty itself.\n> Slow code will have a significant impact in the Faulty performance. Buggy code can bring\n> the Faulty process down.","ref":"Faulty.Ignorer.html"},{"type":"callback","title":"Faulty.Ignorer.ignore?/2","doc":"Decide wether the given error should be ignored or not.\n\nThis function receives both the current Error and context and should return a boolean indicating\nif it should be ignored or not. If the function returns true the error will be ignored, otherwise\nit will be tracked.","ref":"Faulty.Ignorer.html#c:ignore?/2"},{"type":"module","title":"Faulty.Integrations.Oban","doc":"Integration with Oban.","ref":"Faulty.Integrations.Oban.html"},{"type":"module","title":"How to use it - Faulty.Integrations.Oban","doc":"It is a plug and play integration: as long as you have Oban installed the\nFaulty will receive and store the errors as they are reported.","ref":"Faulty.Integrations.Oban.html#module-how-to-use-it"},{"type":"module","title":"How it works - Faulty.Integrations.Oban","doc":"It works using Oban's Telemetry events, so you don't need to modify anything\non your application.\n\n> #### A note on errors grouping {: .warning}\n>\n> All errors reported using `:error` or `{:error, any()}` as the output of\n> your `perform/2` worker function are going to be grouped together (one group\n> of those of errors per worker).\n>\n> The reason of that behaviour is that those errors do not generate an exception,\n> so no stack trace is detected and they are stored as happening in the same\n> place.\n>\n> If you want errors of your workers to be grouped as you may expect on other\n> integrations, you should raise exceptions to report errors instead of gracefully\n> returning an error value.","ref":"Faulty.Integrations.Oban.html#module-how-it-works"},{"type":"module","title":"Default context - Faulty.Integrations.Oban","doc":"By default we store some context for you on errors generated in an Oban\nprocess:\n\n* `job.id`: the unique ID of the job.\n\n* `job.worker`: the name of the worker module.\n\n* `job.queue`: the name of the queue in which the job was inserted.\n\n* `job.args`: the arguments of the job being executed.\n\n* `job.priority`: the priority of the job.\n\n* `job.attempt`: the number of attempts performed for the job.","ref":"Faulty.Integrations.Oban.html#module-default-context"},{"type":"module","title":"Faulty.Integrations.Phoenix","doc":"Integration with Phoenix applications.","ref":"Faulty.Integrations.Phoenix.html"},{"type":"module","title":"How to use it - Faulty.Integrations.Phoenix","doc":"It is a plug and play integration: as long as you have Phoenix installed the\nFaulty will receive and store the errors as they are reported.\n\nIt also collects the exceptions that raise on your LiveView modules.","ref":"Faulty.Integrations.Phoenix.html#module-how-to-use-it"},{"type":"module","title":"How it works - Faulty.Integrations.Phoenix","doc":"It works using Phoenix's Telemetry events, so you don't need to modify\nanything on your application.","ref":"Faulty.Integrations.Phoenix.html#module-how-it-works"},{"type":"module","title":"Errors on the Endpoint - Faulty.Integrations.Phoenix","doc":"This integration only catches errors that raise after the requests hits your\nRouter. That means that an exception on a plug defined on your Endpoint will\nnot be reported.\n\nIf you want to also catch those errors, we recommend you to set up the\n`Faulty.Integrations.Plug` integration too.","ref":"Faulty.Integrations.Phoenix.html#module-errors-on-the-endpoint"},{"type":"module","title":"Default context - Faulty.Integrations.Phoenix","doc":"For errors that are reported when executing regular HTTP requests (the ones\nthat go to Controllers), the context added by default is the same that you\ncan find on the `Faulty.Integrations.Plug` integration.\n\nAs for exceptions generated in LiveView processes, we collect some special\ninformation on the context:\n\n* `live_view.view`: the LiveView module itself,\n\n* `live_view.uri`: last URI that loaded the LiveView (available when the\n`handle_params` function is invoked).\n\n* `live_view.params`: the params received by the LiveView (available when the\n`handle_params` function is invoked).\n\n* `live_view.event`: last event received by the LiveView (available when the\n`handle_event` function is invoked).\n\n* `live_view.event_params`: last event params received by the LiveView\n(available when the `handle_event` function is invoked).","ref":"Faulty.Integrations.Phoenix.html#module-default-context"},{"type":"module","title":"Faulty.Integrations.Plug","doc":"Integration with Plug applications.","ref":"Faulty.Integrations.Plug.html"},{"type":"module","title":"How to use it - Faulty.Integrations.Plug","doc":"","ref":"Faulty.Integrations.Plug.html#module-how-to-use-it"},{"type":"module","title":"Plug applications - Faulty.Integrations.Plug","doc":"The way to use this integration is by adding it to either your `Plug.Builder`\nor `Plug.Router`:\n\n```elixir\ndefmodule MyApp.Router do\n  use Plug.Router\n  use Faulty.Integrations.Plug\n\n  ...\nend\n```","ref":"Faulty.Integrations.Plug.html#module-plug-applications"},{"type":"module","title":"Phoenix applications - Faulty.Integrations.Plug","doc":"There is a particular use case which can be useful when running a Phoenix\nweb application.\n\nIf you want to record exceptions that may occur in your application's endpoint\nbefore reaching your router (for example, in any plug like the ones decoding\ncookies of body contents) you may want to add this integration too:\n\n```elixir\ndefmodule MyApp.Endpoint do\n  use Phoenix.Endpoint\n  use Faulty.Integrations.Plug\n\n  ...\nend\n```","ref":"Faulty.Integrations.Plug.html#module-phoenix-applications"},{"type":"module","title":"Default context - Faulty.Integrations.Plug","doc":"By default we store some context for you on errors generated during a Plug\nrequest:\n\n* `request.host`: the `conn.host` value.\n\n* `request.ip`: the IP address that initiated the request. It includes parsing\nproxy headers\n\n* `request.method`: the HTTP method of the request.\n\n* `request.path`: the path of the request.\n\n* `request.query`: the query string of the request.\n\n* `request.params`: parsed params of the request (only available if they have\nbeen fetched and parsed as part of the Plug pipeline).\n\n* `request.headers`: headers received on the request. All headers are included\nby default except for the `Cookie` ones, as they may include large and\nsensitive content like sessions.","ref":"Faulty.Integrations.Plug.html#module-default-context"},{"type":"module","title":"Faulty.Integrations.Quantum","doc":"Integration with Quantum.","ref":"Faulty.Integrations.Quantum.html"},{"type":"module","title":"How to use it - Faulty.Integrations.Quantum","doc":"It is a plug and play integration: as long as you have Quantum installed the\nFaulty will receive and store the errors as they are reported.","ref":"Faulty.Integrations.Quantum.html#module-how-to-use-it"},{"type":"module","title":"How it works - Faulty.Integrations.Quantum","doc":"It works using Quantum's Telemetry events, so you don't need to modify anything\non your application.","ref":"Faulty.Integrations.Quantum.html#module-how-it-works"},{"type":"function","title":"Faulty.Integrations.Quantum.attach/0","doc":"","ref":"Faulty.Integrations.Quantum.html#attach/0"},{"type":"function","title":"Faulty.Integrations.Quantum.handle_event/4","doc":"","ref":"Faulty.Integrations.Quantum.html#handle_event/4"},{"type":"module","title":"Faulty.Stacktrace","doc":"An Stacktrace contains the information about the execution stack for a given\noccurrence of an exception.","ref":"Faulty.Stacktrace.html"},{"type":"function","title":"Faulty.Stacktrace.new/1","doc":"","ref":"Faulty.Stacktrace.html#new/1"},{"type":"function","title":"Faulty.Stacktrace.source/1","doc":"Source of the error stack trace.\n\nThe first line matching the client application. If no line belongs to the current\napplication, just the first line.","ref":"Faulty.Stacktrace.html#source/1"},{"type":"type","title":"Faulty.Stacktrace.t/0","doc":"","ref":"Faulty.Stacktrace.html#t:t/0"},{"type":"module","title":"Faulty.Stacktrace.Line","doc":"","ref":"Faulty.Stacktrace.Line.html"},{"type":"module","title":"Faulty.Web","doc":"Faulty includes a dashboard to view and inspect errors that occurred\non your application and are already stored in the database.\n\nIn order to use it, you need to add the following to your Phoenix's\n`router.ex` file:\n\n```elixir\ndefmodule YourAppWeb.Router do\n  use Phoenix.Router\n  use Faulty.Web, :router\n\n  ...\n\n  faulty_dashboard \"/errors\"\nend\n```\n\nThis will add the routes needed for Faulty's dashboard to work.","ref":"Faulty.Web.html"},{"type":"module","title":"Security considerations - Faulty.Web","doc":"Errors may contain sensitive information, like IP addresses, users information\nor even passwords sent on forms!\n\nSecuring your dashboard is an important part of integrating Faulty on\nyour project.\n\nIn order to do so, we recommend implementing your own security mechanisms in\nthe form of a mount hook and pass it to the `faulty_dashboard` macro\nusing the `on_mount` option.\n\nYou can find more details on\n`Faulty.Web.Router.faulty_dashboard/2`.","ref":"Faulty.Web.html#module-security-considerations"},{"type":"module","title":"Static assets - Faulty.Web","doc":"Static assets (CSS and JS) are inlined during the compilation. If you have\na [Content Security Policy](https://developer.mozilla.org/en-US/docs/Web/HTTP/CSP)\nbe sure to allow inline styles and scripts.\n\nTo do this, ensure that your `style-src` and `script-src` policies include the\n`unsafe-inline` value.","ref":"Faulty.Web.html#module-static-assets"},{"type":"module","title":"LiveView socket options - Faulty.Web","doc":"By default the library expects you to have your LiveView socket at `/live` and\nusing `websocket` transport.\n\nIf that's not the case, you can configure it adding the following\nconfiguration to your app's config files:\n\n```elixir\nconfig :faulty,\n  socket: [\n    path: \"/my-custom-socket-path\"\n    transport: :longpoll # (accepted values are :longpoll or :websocket)\n  ]\n```","ref":"Faulty.Web.html#module-liveview-socket-options"},{"type":"task","title":"mix faulty.install","doc":"This will inject the necessary dependencies and update your config to start tracking errors with Faulty.","ref":"Mix.Tasks.Faulty.Install.html"},{"type":"task","title":"Example - mix faulty.install","doc":"```bash\nmix faulty.install --env_var TOWERURL\n```","ref":"Mix.Tasks.Faulty.Install.html#module-example"},{"type":"task","title":"Options - mix faulty.install","doc":"* `--env_var` - The environment variable that holds your faulty tower url, defaults to: FAULTY_TOWER_URL","ref":"Mix.Tasks.Faulty.Install.html#module-options"},{"type":"extras","title":"Faulty","doc":"# Faulty\n\nError tracking for your application.","ref":"readme.html"},{"type":"extras","title":"Installation - Faulty","doc":"Add `faulty` to your `mix.exs` file, then `mix deps.get` it.\n\n```elixir\ndef deps do\n  [\n    {:faulty, \"~> 0.1\"}\n  ]\nend\n```\n\nor you can also use `igniter` to add/install `Faulty`.\n\n```elixir\nmix igniter.install faulty`\n```","ref":"readme.html#installation"},{"type":"extras","title":"Configuration - Faulty","doc":"Add the following to your `config/config.exs` file:\n\n```elixir\nconfig :faulty,\n    otp_app: :your_app,\n    enabled: true,\n    retries: 5,\n    connect_options: [...]\n```\n\nThe `:otp_app` option specifies your application, this allows `FaultyTower` to filter only your app's stack traces.\n\nThe `:env` option should be filled in with the name of the environment variable that contains the link to your FaultyTower instance, default is FAULTY_TOWER_URL.\n\nThe `:enabled` option if not given, will default to `true`. You probable want to turn this off for your test environment.\n\nThe `:retries` option is used to tell `Req` how many times to retry sending the error to `FaultyTower`, defaults to 5. If it hasn't succeeded by then the error will be dropped.\n\nThe `:connect_options` are passed through to `Req`.","ref":"readme.html#configuration"},{"type":"extras","title":"Error tracking - Faulty","doc":"Once configured `Faulty` is ready to start tracking your errors. It automatically starts with your application and tracks errors in your Phoenix controllers, LiveViews en Oban jobs.\nCheckout the `Faulty.Integrations.Phoenix` and `Faulty.Integrations.Oban` for more detailed information.\n\nIf your application uses Plug but not Phoenix, you will need to add the relevant integration in your `Plug.Builder` or `Plug.Router` module.\n\n```elixir\ndefmodule MyApp.Router do\n  use Plug.Router\n  use Faulty.Integrations.Plug\n\n  # Your code here\nend\n```\n\nThis is also required if you want to track errors that happen in your Phoenix endpoint, before the Phoenix router starts handling the request. Keep in mind that this won't be needed in most cases as endpoint errors are infrequent.\n\n```elixir\ndefmodule MyApp.Endpoint do\n  use Phoenix.Endpoint\n  use Faulty.Integrations.Plug\n\n  # Your code here\nend\n```\n\nYou can learn more about this in the `Faulty.Integrations.Plug` module documentation.","ref":"readme.html#error-tracking"},{"type":"extras","title":"Error context - Faulty","doc":"The default integrations include some additional context when tracking errors. You can take a look at the relevant integration modules to see what is being tracked out of the box.\n\nIn certain cases, you may want to include some additional information when tracking errors. For example it may be useful to track the user ID that was using the application when an error happened. Fortunately, Faulty allows you to enrich the default context with custom information.\n\nThe `Faulty.set_context/1` function stores the given context in the current process so any errors that occur in that process (for example, a Phoenix request or an Oban job) will include this given context along with the default integration context.\n\nThere are some requirements on the type of data that can be included in the context, so we recommend taking a look at `Faulty.set_context/1` documentation\n\n```elixir\nFaulty.set_context(%{\"user_id\" =>  conn.assigns.current_user.id})\n```\n\nYou may also want to sanitize or filter out some information from the context before saving it. To do that you can take a look at the `Faulty.Filter` behaviour.","ref":"readme.html#error-context"},{"type":"extras","title":"Manual error tracking - Faulty","doc":"If you want to report custom errors that fall outside the default integration scope, you may use `Faulty.report/2`. This allows you to report an exception yourself:\n\n```elixir\ntry do\n  # your code\ncatch\n  e ->\n    Faulty.report(e, __STACKTRACE__)\nend\n```\n\nYou can also use `Faulty.report/3` and set some custom context that will be included along with the reported error.","ref":"readme.html#manual-error-tracking"},{"type":"extras","title":"Ignoring errors - Faulty","doc":"Faulty tracks every error by default. In certain cases some errors may be expected or just not interesting to track.\nFaulty provides functionality that allows you to ignore errors based on their attributes and context.\n\nTake a look at the `Faulty.Ignorer` behaviour for more information about how to implement your own ignorer.","ref":"readme.html#ignoring-errors"}],"proglang":"elixir","content_type":"text/markdown","producer":{"name":"ex_doc","version":"0.37.3"}}